︠afb1cc51-6809-4be9-ba14-99ec8b3c7699i︠
%html
<p>We are going to generate a Sierpinski Triangle using an "L-system" method. An L-system can use a string of characters (which we will call the "genome") as instructions for drawing a picture. It starts out with a simple string (we call it the "start"), which is composed of some of the available instructions, and then uses a set of rules to modify the string over a series of iterations.</p>
<p>For example, if our instructions are "A" and "B" (where each letter draws something different), and our starting string is "AB" and we have one rule which specifies that each time we see the letter "B" we replace it with the string "BA," then after one iteration, "AB" will become "ABA". After another iteration, it will become "ABAA", since it replaces the "B" in "ABA" with "BA".</p>
<p>For our purposes, we will first set up the following instructions for use in the L-system calculations:</p>
<ul>
<li>F means "draw forward."</li>
<li>G means "draw forward" as well. We need a separate letter for it because one of our replacement rules makes a distinction between F and G.</li>
<li>+ means "turn left by angle."</li>
<li>- means "turn right by angle."</li>
</ul>
<p>We will also use an angle of 60&deg;.</p>
<div id="_mcePaste" style="overflow: hidden; position: absolute; left: -10000px; top: 0px; width: 1px; height: 1px;">(which we will call the "genome")</div>

︡d6c71b0e-83e9-44da-a78e-96e4bbb2b066︡{"html": "<p>We are going to generate a Sierpinski Triangle using an \"L-system\" method. An L-system can use a string of characters (which we will call the \"genome\") as instructions for drawing a picture. It starts out with a simple string (we call it the \"start\"), which is composed of some of the available instructions, and then uses a set of rules to modify the string over a series of iterations.</p>\n<p>For example, if our instructions are \"A\" and \"B\" (where each letter draws something different), and our starting string is \"AB\" and we have one rule which specifies that each time we see the letter \"B\" we replace it with the string \"BA,\" then after one iteration, \"AB\" will become \"ABA\". After another iteration, it will become \"ABAA\", since it replaces the \"B\" in \"ABA\" with \"BA\".</p>\n<p>For our purposes, we will first set up the following instructions for use in the L-system calculations:</p>\n<ul>\n<li>F means \"draw forward.\"</li>\n<li>G means \"draw forward\" as well. We need a separate letter for it because one of our replacement rules makes a distinction between F and G.</li>\n<li>+ means \"turn left by angle.\"</li>\n<li>- means \"turn right by angle.\"</li>\n</ul>\n<p>We will also use an angle of 60&deg;.</p>\n<div id=\"_mcePaste\" style=\"overflow: hidden; position: absolute; left: -10000px; top: 0px; width: 1px; height: 1px;\">(which we will call the \"genome\")</div>"}︡
︠30053ab2-4b67-460b-8c42-0417564d29d2︠
ANGLE = 60

F = "F"
G = "G"
P = "+"
M = "-"
︡2a71f4a9-6d15-4e14-b7ba-fc4016d0fad5︡︡
︠642a4187-bf49-4439-806b-1d9742e35fe2i︠
%html
<p>For convenience, let's define which of the instructions are constants (are never replaced by any rule), and which are variables (are sometimes replaced by a rule):</p>
<div id="_mcePaste" style="overflow: hidden; position: absolute; left: -10000px; top: 0px; width: 1px; height: 1px;"><!-- 		@page { size: 8.5in 11in; margin: 0.79in } 		P { margin-bottom: 0.08in } -->
<p>convenience</p>
</div>

︡7209af24-56b4-4713-943b-07ae20a45fba︡{"html": "<p>For convenience, let's define which of the instructions are constants (are never replaced by any rule), and which are variables (are sometimes replaced by a rule):</p>\n<div id=\"_mcePaste\" style=\"overflow: hidden; position: absolute; left: -10000px; top: 0px; width: 1px; height: 1px;\"><!-- \t\t@page { size: 8.5in 11in; margin: 0.79in } \t\tP { margin-bottom: 0.08in } -->\n<p>convenience</p>\n</div>"}︡
︠d5b32176-c6d0-4ee2-91f5-8eb90b67b8e2︠
CONSTS = (P, M)
VARS = (F, G)
︡c94084d7-3497-4d7e-b215-773407c1f97f︡︡
︠1889d1e0-728b-4694-865e-a23132bd3aafi︠
%html
<p>Next, we define the generator rules.</p>
<p>For each tuple, the first value is the pattern to replace, and the second value is the pattern to replace it with.</p>
<p>The comment after the line should make the replacement clearer. It is of the form:</p>
<p><span style="font-family: courier new,courier;">PATTERN_TO_REPLACE -&gt; REPLACEMENT_PATTERN</span></p>

︡ae005710-d0bd-4c55-a1a5-b22f05873a16︡{"html": "<p>Next, we define the generator rules.</p>\n<p>For each tuple, the first value is the pattern to replace, and the second value is the pattern to replace it with.</p>\n<p>The comment after the line should make the replacement clearer. It is of the form:</p>\n<p><span style=\"font-family: courier new,courier;\">PATTERN_TO_REPLACE -&gt; REPLACEMENT_PATTERN</span></p>"}︡
︠a108a50b-fd54-4e12-b444-213c87360f0a︠
rules = (
    (F, G + M + F + M + G), # F -> G-F-G
    (G, F + P + G + P + F)) # G -> F+G+F
︡90d546df-4b85-402e-870f-95545b644354︡︡
︠50b3e955-5858-4001-af52-372e716b94b5i︠
%html
<p>Finally, we define the start state. In our case, it's just going to be a simple "go forward" instruction:</p>

︡ad6e8c91-6d72-4ab5-ad73-1cadce4d762e︡{"html": "<p>Finally, we define the start state. In our case, it's just going to be a simple \"go forward\" instruction:</p>"}︡
︠169d86c0-12b6-4f0f-a140-19a95e378698︠
start = F
︡14a7740a-7b1e-4c7f-8122-d0bab30c3c21︡︡
︠ca3b3e6e-dd64-4709-ba37-ded385bcf66fi︠
%html
<p>Before going any further, let's define the drawing algorithm and test it using the starting state.</p>
<p><span style="font-family: courier new,courier;">update_angle </span>will take one of the angle movement instructions and modify which direction the line is drawn based on instruction.</p>
<p><span style="font-family: courier new,courier;">next_point </span>takes the current angle and the current point, and uses that to draw the next point. Note that we have to a) convert from polar to rectangular coords (hence the trig functions), and also from degrees to radians (hence the math.pi / 180).</p>
<p><span style="font-family: courier new,courier;">generate_lines </span>parses the instructions in the genome and converts them into points and lines.</p>

︡e8ecee45-d95a-4df5-8317-9e24e05d23f1︡{"html": "<p>Before going any further, let's define the drawing algorithm and test it using the starting state.</p>\n<p><span style=\"font-family: courier new,courier;\">update_angle </span>will take one of the angle movement instructions and modify which direction the line is drawn based on instruction.</p>\n<p><span style=\"font-family: courier new,courier;\">next_point </span>takes the current angle and the current point, and uses that to draw the next point. Note that we have to a) convert from polar to rectangular coords (hence the trig functions), and also from degrees to radians (hence the math.pi / 180).</p>\n<p><span style=\"font-family: courier new,courier;\">generate_lines </span>parses the instructions in the genome and converts them into points and lines.</p>"}︡
︠1e4e7c63-75be-48cc-b1b1-f85290564cb9︠
def update_angle(c, angle):
    if c == P:
        angle = angle + ANGLE
    if c == M:
        angle = angle - ANGLE
    return angle
    
def next_point(point, angle):
    x = point[0] + math.cos(angle * (math.pi / 180))
    y = point[1] + math.sin(angle * (math.pi / 180))
    return (x, y)

def generate_lines(genome):
    angle = 0
    point = (0, 0)
    points = [point]
    
    for c in genome:
        if c in CONSTS:
            angle = update_angle(c, angle)
        if c in VARS:
            point = next_point(point, angle)
            points.append(point)
    return line(points)

lines = generate_lines(start)
lines.plot()
︡52e56bf2-d710-4cc6-8580-8dcd2b256b35︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠fd0906f2-c646-41d5-8e24-2a0d11a851cbi︠
%html
<p>Our starting instruction does nothing but go forward, so that looks about right.</p>
<p>Next, let's define the function that does the actual replacements:</p>

︡d2aacb68-0a86-4888-b661-1e123a3de163︡{"html": "<p>Our starting instruction does nothing but go forward, so that looks about right.</p>\n<p>Next, let's define the function that does the actual replacements:</p>"}︡
︠03f72146-5772-4f63-b48f-9f752b365ec1︠
def replace(genome, rules):
    new_genome = ""
    for c in genome:
        replacement_made = False
        for rule in rules:
            if rule[0] == c:
                new_genome = new_genome + rule[1]
                replacement_made = True
        if (not replacement_made):
            new_genome = new_genome + c
    return new_genome
    
def test_replace():
    expected = "G-F-G";
    print "Expected Result: " + expected
    actual = replace(start, rules)
    print "Actual Result: " + actual
    if expected == actual:
        print "The replacement algorithm seems to be working."
    else:
        print "Something's wrong with the replacement algorithm."

test_replace()
︡2c72e34c-dc56-415b-ae30-01cb3e155271︡{"stdout": "Expected Result: G-F-G\nActual Result: G-F-G\nThe replacement algorithm seems to be working."}︡
︠ade75e00-529d-41aa-9004-2f05dcbc11d1i︠
%html
<p>Ok, so now that that's working, we just need to apply the replacement rules to the genome for an arbitrary number of iterations.</p>

︡cbd37f10-12fd-42da-9613-11ccd1e33ab4︡{"html": "<p>Ok, so now that that's working, we just need to apply the replacement rules to the genome for an arbitrary number of iterations.</p>"}︡
︠db47e983-3a33-40b2-8f93-b6c993535a5b︠
def l_generate(start, rules, iterations):
    genome = start
    for i in range(0, iterations):
        genome = replace(genome, rules)
    return genome
︡a270ccc7-3fdc-48d5-ab79-4075956bbc8e︡︡
︠36257613-ee80-402f-bc05-dd0475eeabe6i︠
%html
<p>Now that everything's set up, let's plot the Sierpinski Triangles generated by a few different iteration values. We'll use 2, 4, 6, and 8. Note that the odd-numbered iterations don't actually generate the triangles: they are just steps to get to the even numbered iterations.</p>
<p>10 iterations creates a very smooth triangle, but it takes a while to calculate.</p>

︡b9f0f3b5-04b9-45fb-bf39-37a809ea54c0︡{"html": "<p>Now that everything's set up, let's plot the Sierpinski Triangles generated by a few different iteration values. We'll use 2, 4, 6, and 8. Note that the odd-numbered iterations don't actually generate the triangles: they are just steps to get to the even numbered iterations.</p>\n<p>10 iterations creates a very smooth triangle, but it takes a while to calculate.</p>"}︡
︠7d27d36e-83b7-46d9-80d7-81e2f7c9c5f2︠
genome = l_generate(start, rules, 2)
lines = generate_lines(genome)
lines.plot()
︡1fb3c73c-2f24-48f9-adca-fe1dcdcb21d2︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠da048241-f107-4aa2-8ab0-9aebf86aed1a︠
genome = l_generate(start, rules, 4)
lines = generate_lines(genome)
lines.plot()
︡5d299639-4828-40f3-bd0f-66b9e8f6747f︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠d441a537-f017-43fa-a28e-a4da0785d860︠
genome = l_generate(start, rules, 6)
lines = generate_lines(genome)
lines.plot()
︡88a08f20-3960-4d4d-bffb-fdd1efd07c60︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
︠19e05671-d60c-4c85-85e3-6d10e71d783c︠
genome = l_generate(start, rules, 8)
lines = generate_lines(genome)
lines.plot()
︡0287ab8d-ea63-4f2d-b310-7fc2ffbee579︡{"html": "<font color='black'><img src='cell://sage0.png'></font>"}︡
